module DkComposer
  class Stack
    include Helper
    attr_accessor :name, :services, :version, :volumes, :networks
    def initialize(name)
      @name = name
      @services = []
      @volumes = {}
      @networks = {}
      @version = '2'
    end

    def service(name, **opts, &block)
      ret = if SERVICE[name]
              SERVICE[name].deep_clone
            else
              Service.create(name)
            end
      ret.config(opts)
      ret.instance_exec(&block) if block
      @services << ret
      ret
    end

    def volume(name, **opts, &block)
      ret = if VOLUME[name]
              VOLUME[name].deep_clone
            else
              Volume.create(name)
            end
      ret.config(opts)
      ret.instance_exec(&block) if block
      @volumes[name] = ret
      ret
    end

    def network(name, **opts, &block)
      ret = if NETWORK[name]
              NETWORK[name].deep_clone
            else
              Network.create(name)
            end
      ret.config(opts)
      ret.instance_exec(&block) if block
      @networks[name] = ret
      ret
    end

    def version(value = nil)
      value ? @version = value : @version
    end

    def run(*opts, path: build_path)
      composefile = tempfile(to_yaml, 'docker-compose.yml', path)
      up_cmd = [
        "pushd #{path}",
        "docker-compose -f #{composefile}  up #{opts.join(' ')}",
        'popd'
      ].join('&&')
      run_command(up_cmd)
    end

    def stop(*opts, path: build_path)
      composefile = tempfile(to_yaml, 'docker-compose.yml', path)
      up_cmd = [
        "pushd #{path}",
        "docker-compose -f #{composefile}  down #{opts.join(' ')}",
        'popd'
      ].join('&&')
      run_command(up_cmd)
    end

    def to_h
      ret = {
        version: @version,
        volumes: @volumes.each_with_object({}) do |(_, v), r|
          r.merge! v.to_h
        end.stringfy_key,
        networks: @networks.each_with_object({}) do |(_, v), r|
          r.merge! v.to_h
        end.stringfy_key,
        services: @services.each_with_object({}) do |s, r|
          r.merge! s.to_h
        end
      }
      ret
    end

    def to_yaml
      Hash[to_h.collect { |k, v| [k.to_s, v] }].to_yaml
    end

    def self.create(name, *params, **opts, &block)
      ret = Stack.new(name)
      ret.config(opts)
      ret.instance_exec(*params, &block) if block
      ret
    end

    def to_s
      ret = <<-DUMP
stack(:#{name}, version: '#{version}',build_path:"#{build_path}") do
#{@volumes.map { |(_, v)| v.to_s }.join("\n")}
#{@networks.map { |(_, v)| v.to_s }.join("\n")}
#{@services.map(&:to_s).join("\n")}
end
    DUMP
      ret
    end
    alias dump to_s

    def from_h(h)
      version h[:version]

      h[:volumes].each do |k, v|
        volume(k, v || {})
      end
      h[:networks].each do |k, v|
        network(k, v || {})
      end

      h[:services].each do |k, v|
        service(k, v)
      end
    end

    def self.load_compose_file(file, name = :auto_generated)
      opts = YAML.load_file(file)
      ret = create(name)
      ret.from_h(opts.symbolize_key)
      ret
    end
  end
end
def stack(name, *params, **opts, &block)
  ret = DkComposer::Stack.create(name, *params, **opts, &block)
  shortdesc, longdesc = (@desc || []).pop
  ret.shortdesc = shortdesc
  ret.longdesc = longdesc
  DkComposer::STACK[name] = ret
end
